/* version.h: version information for GNU bc and GNU dc */

/* This file is part of GNU bc and GNU dc.
 * Copyright (C) 1994, 1997, 1998 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can either send email to this
 * program's author (see below) or write to: The Free Software Foundation,
 * Inc.; 675 Mass Ave. Cambridge, MA 02139, USA.
 */

#define BC_VERSION  \
"bc 1.05\n\
Copyright 1991, 1992, 1993, 1994, 1997, 1998 Free Software Foundation, Inc."

#define DC_VERSION \
"dc 1.2 (GNU bc 1.05)\n\
Copyright 1994, 1997, 1998 Free Software Foundation, Inc."
